import java.io.IOException;
import java.util.Map;
import java.util.Set;

/**
 * Rozhrani pro nacitani a zapis informaci o mestech a jejich navrhovatelich
 * (urbanistu) z/do souboru.
 * 
 * @author Ales Zlamal
 * @author Radek Oslejsek
 * @version 2008-01-18
 */

public interface TownStorage
{
    /**
     * Metoda nacte ze vstupniho souboru informace o navrhovatelich a jejich 
     * mestech. Format vstupniho souboru (viz take soubor input.txt):
     * 
     * a) kazdy radek s vztahuje vzdy k jednomu navrhovateli
     * b) na zacatku radku je jmeno navrhovatele, pak dvojtecka a pak
     *    jmena mest, ktera navrhoval, oddelena carkou
     * c) kazdy navrhovat ma v souboru pouze jeden zaznam (radek)
     * d) jedno mesto mohlo byt navrzeno vice lidmi, tj. muze se objevit
     *    na nekolika radcich
     * 
     * Pr. formatu:
     * Marek:Firenze,Orvieto,Arezzo,Siena,Urbino,Mantua,Roma
     * Adam:Firenze,Pisa,San Gimignano,Roma,Siena,Orvieto
     * 
     * @param fileName Nazev vstupniho souboru
     * @return mapa, kde klicem je navrhovatel a hodnotou je mnozina
     *         vsech mest, ktere navrhoval
     * @throws NullPointerException pokud je fileName null
     * @throws TownException pri jajekoliv chybe 
     *         vstupu/vystupu, nebo pri chybnem formatu vstupniho souboru
     */
    Map<Person,Set<Town>> read(String fileName) throws TownException;
    
    /**
     * @param townsOfProp mapa navrhovatelu (klic) a jejich mest (hodnota), 
     *        tj. vystup metody read()
     * @return mapa, kde klicem je mesto a hodnotou je mnozina vsech 
     *         navrhovatelu, kteri se na vystavbe mesta podileli
     * @throws NullPointerException pokud je townsOfProp null
     */
    Map<Town,Set<Person>> proposalsOfTowns(Map<Person,Set<Town>> townsOfProp);
    
    /**
     * Metoza zapise informace o mestech a jejich navrhovatelich do 
     * textoveho souboru v nasledujicim formatu
     * (viz take ocekavany obsah souboru output.txt v zadani)
     * 
     * a) kazdy radek s vztahuje vzdy k jednomu mestu
     * b) na zacatku radku je nazev mesta, pak dvojtecka a pak
     *    jmena navrhovatelu, kteri se na vystavbe mesta podileli
     * c) kazde mesto ma v souboru pouze jeden zaznam (radek)
     * d) jeden navrhovatel se mohl podilet na vistavbe nekolika mest;
     *    proto se muze objevit na vice radcich
     *    
     * Pr. formatu:
     * 
     * 
     * @param propOfTowns mapa mest (klic) a jejich navrhovatelu (hodnota),
     *        tj. vystup metody proposalsOfTowns()
     * @param fileName nazev vystupniho souboru
     * @throws NullPointerException pokud je propOfTowns nebo fileName null
     * @throws TownException pri jakekoliv chybe vstupu/vystupu
     */
    void write(Map<Town,Set<Person>> propOfTowns, String fileName) 
        throws TownException;    
}
